/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.contact.dao;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.ArrayList;

import sync4j.exchange.httptransport.WebDavHttpTransport;
import sync4j.exchange.items.contact.model.Contact;
import sync4j.exchange.items.common.dao.ItemDAO;
import sync4j.exchange.xml.XmlParseException;
import sync4j.exchange.xml.XmlParser;

import sync4j.exchange.DataAccessException;

import sync4j.framework.server.store.NotFoundException;

/*
 * This class implements methods to access contacts data
 * in exchange server datastore
 *
 * @author Fabio Maggi @ Funambol
 * @version $Id: ContactDAO.java,v 1.9.2.2 2005/04/15 09:18:40 nichele Exp $
 *
 **/
public class ContactDAO extends ItemDAO {

    //---------------------------------------------------------------- Constants

    private static final String WEBDAV_MSG_PROPPATCH_UPDATE_CONTACT        =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                        +
    "<D:propertyupdate xmlns:D=\"DAV:\" "                                  +
    "xmlns:EX=\"http://schemas.microsoft.com/exchange/\" "                 +
    "xmlns:HM=\"urn:schemas:contacts:\" "                                  +
    "xmlns:HG=\"urn:schemas:calendar:\" "                                  +
    "xmlns:HN=\"urn:schemas:httpmail:\" "                                  +
    "xmlns:HO=\"urn:schemas:mailheader:\">\n"                              +
    "<D:set>\n"                                                            +
    "<D:prop>\n"                                                           +
    "<D:contentclass>urn:content-classes:person</D:contentclass>\n"        +
    "<EX:outlookmessageclass>IPM.Contact</EX:outlookmessageclass>\n"       +
    "<HM:cn>{0}</HM:cn>\n"                                                 +
    "<HM:fileas>{1}</HM:fileas>\n"                                         +
    "<HN:subject>{2}</HN:subject>\n"                                       +
    "<HO:subject>{3}</HO:subject>\n"                                       +
    "<HM:givenName>{4}</HM:givenName>\n"                                   +
    "<HM:sn>{5}</HM:sn>\n"                                                 +
    "<HM:middlename>{6}</HM:middlename>\n"                                 +
    "<HM:title>{7}</HM:title>\n"                                           +
    "<HM:o>{8}</HM:o>\n"                                                   +
    "<HM:homeStreet>{9}</HM:homeStreet>\n"                                 +
    "<HM:homeCity>{10}</HM:homeCity>\n"                                    +
    "<HM:homeState>{11}</HM:homeState>\n"                                  +
    "<HM:homePostalCode>{12}</HM:homePostalCode>\n"                        +
    "<HM:homeCountry>{13}</HM:homeCountry>\n"                              +
    "<HM:street>{14}</HM:street>\n"                                        +
    "<HM:l>{15}</HM:l>\n"                                                  +
    "<HM:st>{16}</HM:st>\n"                                                +
    "<HM:postalcode>{17}</HM:postalcode>\n"                                +
    "<HM:co>{18}</HM:co>\n"                                                +
    "<HM:otherstreet>{19}</HM:otherstreet>\n"                              +
    "<HM:othercity>{20}</HM:othercity>\n"                                  +
    "<HM:otherstate>{21}</HM:otherstate>\n"                                +
    "<HM:otherpostalcode>{22}</HM:otherpostalcode>\n"                      +
    "<HM:othercountry>{23}</HM:othercountry>\n"                            +
    "<HM:telephoneNumber>{24}</HM:telephoneNumber>\n"                      +
    "<HM:homePhone>{25}</HM:homePhone>\n"                                  +
    "<HM:mobile>{26}</HM:mobile>\n"                                        +
    "<HM:secretaryphone>{27}</HM:secretaryphone>\n"                        +
    "<HM:telephonenumber2>{28}</HM:telephonenumber2>\n"                    +
    "<HM:facsimiletelephonenumber>{29}</HM:facsimiletelephonenumber>\n"    +
    "<HM:callbackphone>{30}</HM:callbackphone>\n"                          +
    "<HM:othermobile>{31}</HM:othermobile>\n"                              +
    "<HM:organizationmainphone>{32}</HM:organizationmainphone>\n"          +
    "<HM:homephone2>{33}</HM:homephone2>\n"                                +
    "<HM:homefax>{34}</HM:homefax>\n"                                      +
    "<HM:internationalisdnnumber>{35}</HM:internationalisdnnumber>\n"      +
    "<HM:otherTelephone>{36}</HM:otherTelephone>\n"                        +
    "<HM:otherfax>{37}</HM:otherfax>\n"                                    +
    "<HM:pager>{38}</HM:pager>\n"                                          +
    "<HM:businesshomepage>{39}</HM:businesshomepage>\n"                    +
    "<HM:department>{40}</HM:department>\n"                                +
    "<HM:roomnumber>{41}</HM:roomnumber>\n"                                +
    "<HM:profession>{42}</HM:profession>\n"                                +
    "<HM:nickname>{43}</HM:nickname>\n"                                    +
    "<HM:spousecn>{44}</HM:spousecn>\n"                                    +
    "<HM:manager>{45}</HM:manager>\n"                                      +
    "<HM:secretarycn>{46}</HM:secretarycn>\n"                              +
    "<HG:fburl>{47}</HG:fburl>\n"                                          +
    "<HM:email1>{48}</HM:email1>\n"                                        +
    "<HM:email2>{49}</HM:email2>\n"                                        +
    "<HM:email3>{50}</HM:email3>\n"                                        +
    "<HM:fileas>{51}</HM:fileas>\n"                                        +
    "</D:prop>\n"                                                          +
    "</D:set>\n"                                                           +
    "</D:propertyupdate>"                                                  ;

    private static final String WEBDAV_MSG_SELECT_CONTACTS                 =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                        +
    "<D:searchrequest xmlns:D =\"DAV:\">\n"                                +
    "<D:sql>\n"                                                            +
    "Select "                                                              +
    "\"http://schemas.microsoft.com/repl/repl-uid\" AS repluid, "          +
    "\"urn:schemas:contacts:sn\" AS lastname, "                            +
    "\"urn:schemas:contacts:givenName\" AS givenname, "                    +
    "\"urn:schemas:contacts:middlename\" AS middlename, "                  +
    "\"urn:schemas:contacts:title\" AS title, "                            +
    "\"urn:schemas:contacts:o\" AS organization, "                         +
    "\"urn:schemas:contacts:homeStreet\" AS homestreet, "                  +
    "\"urn:schemas:contacts:homeCity\" AS homecity, "                      +
    "\"urn:schemas:contacts:homeState\" AS homestate, "                    +
    "\"urn:schemas:contacts:homePostalCode\" AS homepostalcode, "          +
    "\"urn:schemas:contacts:homeCountry\" AS homecountry, "                +
    "\"urn:schemas:contacts:street\" AS street, "                          +
    "\"urn:schemas:contacts:l\" AS city, "                                 +
    "\"urn:schemas:contacts:st\" AS state, "                               +
    "\"urn:schemas:contacts:postalcode\" AS postalcode, "                  +
    "\"urn:schemas:contacts:co\" AS country, "                             +
    "\"urn:schemas:contacts:otherstreet\" AS otherstreet, "                +
    "\"urn:schemas:contacts:othercity\" AS othercity, "                    +
    "\"urn:schemas:contacts:otherstate\" AS otherstate, "                  +
    "\"urn:schemas:contacts:otherpostalcode\" AS otherpostalcode, "        +
    "\"urn:schemas:contacts:othercountry\" AS othercountry, "              +
    "\"urn:schemas:contacts:telephoneNumber\" AS telephonenumber, "        +
    "\"urn:schemas:contacts:homePhone\" AS homephone, "                    +
    "\"urn:schemas:contacts:mobile\" AS mobile, "                          +
    "\"urn:schemas:contacts:secretaryphone\" AS secretaryphone, "          +
    "\"urn:schemas:contacts:telephonenumber2\" AS telephonenumber2, "      +
    "\"urn:schemas:contacts:facsimiletelephonenumber\" "                   +
            "AS facsimiletelephonenumber, "                                +
    "\"urn:schemas:contacts:callbackphone\" AS callbackphone, "            +
    "\"urn:schemas:contacts:othermobile\" AS othermobile, "                +
    "\"urn:schemas:contacts:organizationmainphone\" "                      +
            "AS organizationmainphone, "                                   +
    "\"urn:schemas:contacts:homephone2\" AS homephone2, "                  +
    "\"urn:schemas:contacts:homefax\" AS homefax, "                        +
    "\"urn:schemas:contacts:internationalisdnnumber\" "                    +
            "AS internationalisdnnumber, "                                 +
    "\"urn:schemas:contacts:otherTelephone\" AS othertelephone, "          +
    "\"urn:schemas:contacts:otherfax\" AS otherfax, "                      +
    "\"urn:schemas:contacts:pager\" AS pager, "                            +
    "\"urn:schemas:contacts:businesshomepage\" AS businesshomepage, "      +
    "\"urn:schemas:contacts:department\" AS department, "                  +
    "\"urn:schemas:contacts:roomnumber\" AS roomnumber, "                  +
    "\"urn:schemas:contacts:profession\" AS profession, "                  +
    "\"urn:schemas:contacts:nickname\" AS nickname, "                      +
    "\"urn:schemas:contacts:spousecn\" AS spousecn, "                      +
    "\"urn:schemas:contacts:manager\" AS manager, "                        +
    "\"urn:schemas:contacts:secretarycn\" AS secretarycn, "                +
    "\"urn:schemas:calendar:fburl\" AS fburl, "                            +
    "\"urn:schemas:contacts:email1\" AS email1, "                          +
    "\"urn:schemas:contacts:email2\" AS email2, "                          +
    "\"urn:schemas:contacts:email3\" AS email3, "                          +
    "\"urn:schemas:contacts:fileas\" AS fileas, "                          +
    "\"DAV:displayname\", "                                                +
    "\"DAV:getlastmodified\" AS getlastmodified, "                         +
    "\"DAV:isfolder\" AS isfolder "                                        +
    "FROM \"/{0}/{1}/{2}\"  "                                              +
    "{3} "                                                                 +
    "</D:sql>\n"                                                           +
    "</D:searchrequest>"                                                   ;


    private static final String TAG_EMAIL1            = "email1"           ;
    private static final String TAG_EMAIL2            = "email2"           ;
    private static final String TAG_EMAIL3            = "email3"           ;

    private static final String TAG_FILEAS            = "fileas"           ;

    private static final String TAG_LAST_NAME         = "lastname"         ;
    private static final String TAG_FIRST_NAME        = "givenname"        ;
    private static final String TAG_MIDDLE_NAME       = "middlename"       ;
    private static final String TAG_TITLE             = "title"            ;
    private static final String TAG_ORGANIZATION      = "organization"     ;

    private static final String TAG_HOME_STREET       = "homestreet"       ;
    private static final String TAG_HOME_CITY         = "homecity"         ;
    private static final String TAG_HOME_STATE        = "homestate"        ;
    private static final String TAG_HOME_POSTAL_CODE  = "homepostalcode"   ;
    private static final String TAG_HOME_COUNTRY      = "homecountry"      ;

    private static final String TAG_STREET            = "street"           ;
    private static final String TAG_CITY              = "city"             ;
    private static final String TAG_STATE             = "state"            ;
    private static final String TAG_POSTALCODE        = "postalcode"       ;
    private static final String TAG_COUNTRY           = "country"          ;

    private static final String TAG_OTHER_STREET      = "otherstreet"      ;
    private static final String TAG_OTHER_CITY        = "othercity"        ;
    private static final String TAG_OTHER_STATE       = "otherstate"       ;
    private static final String TAG_OTHER_POSTALCODE  = "otherpostalcode"  ;
    private static final String TAG_OTHER_COUNTRY     = "othercountry"     ;

    private static final String TAG_TELEPHONE_NUMBER  = "telephonenumber"  ;
    private static final String TAG_HOME_PHONE        = "homephone"        ;
    private static final String TAG_MOBILE            = "mobile"           ;
    private static final String TAG_SECRETARY_PHONE   = "secretaryphone"   ;
    private static final String TAG_TELEPHONE_NUMBER2 = "telephonenumber2" ;
    private static final String
        TAG_FAC_SIMILE_TELEPHONE_NUMBER
            = "facsimiletelephonenumber"                                   ;
    private static final String TAG_CALL_BACK_HOME    = "callbackphone"    ;
    private static final String TAG_OTHER_MOBILE      = "othermobile"      ;
    private static final String
        TAG_ORGANIZATION_MAIN_PHONE
            = "organizationmainphone"                                      ;
    private static final String TAG_HOME_PHONE2       = "homephone2"       ;
    private static final String TAG_HOME_FAX          = "homefax"          ;
    private static final String
        TAG_INTERNATION_ISDN_NUMBER
        = "internationalisdnnumber"                                        ;
    private static final String TAG_OTHER_TELEPHONE
        = "othertelephone"                                                 ;
    private static final String TAG_OTHER_FAX        = "otherfax"          ;
    private static final String TAG_PAGER            = "pager"             ;

    private static final String
        TAG_BUSINESS_HOME_PAGE
        = "businesshomepage"                                               ;

    private static final String TAG_DEPARTMENT      = "department"         ;
    private static final String TAG_ROOMNUMBER      = "roomnumber"         ;
    private static final String TAG_PROFESSION      = "profession"         ;
    private static final String TAG_NICKNAME        = "nickname"           ;
    private static final String TAG_SPOUSECN        = "spousecn"           ;
    private static final String TAG_MANAGER         = "manager"            ;
    private static final String TAG_SECRETARY       = "secretarycn"        ;
    private static final String TAG_BURL            = "fburl"              ;

    private static final String TAG_RESPONSE        = "a:response"         ;

    private static final String TAG_REPLUID         = "repluid"            ;

    private static final String TAG_LAST_MODIFIED   = "getlastmodified"    ;

    //------------------------------------------------------------- Private data



    private WebDavHttpTransport  webDavHttp          = null ;

    //------------------------------------------------------------- Costructors

    public ContactDAO(String  exchangeServerHost ,
                      int     exchangeServerPort)
        throws DataAccessException {

        this.webDavHttp         = new WebDavHttpTransport(exchangeServerHost,
                                                          exchangeServerPort);
    }

    //------------------------------------------------------------- Public methods

    /**
     * insert / update contact<br>
     *
     * @param c
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new / updated contact
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact setContact(Contact c              ,
                              String  username       ,
                              String  credentials    ,
                              String  exchangeFolder )
        throws DataAccessException {

        String response                = null ;

        String id                      = null ;

        String webDavContactMsg        = null ;
        String webDavHeaderMsg         = null ;

        String firstName               = null ;
        String lastName                = null ;
        String middleName              = null ;

        String server                  = null ;
        String resource                = null ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        XmlParser p = new XmlParser();

        firstName  = p.getString(c.getFirstName  () ) ;
        lastName   = p.getString(c.getLastName   () ) ;
        middleName = p.getString(c.getMiddleName () ) ;

        webDavContactMsg

            = MessageFormat.format(WEBDAV_MSG_PROPPATCH_UPDATE_CONTACT,
              new Object[] {
                c.getFirstName()                                          +
                (((middleName.length() > 0) && (firstName.length()> 0))
                  ? " " : "")                                             +
                middleName                                                +
                (((lastName.length() > 0) && (middleName.length() > 0))
                  ? " " : "")                                             +
                lastName                                                   ,
                lastName                                                  +
                (((firstName.length() > 0) && (lastName.length() > 0))
                    ? ", " : ",")                                         +
                firstName                                                 +
                (((middleName.length() > 0) && (firstName.length() > 0))
                    ? " "  : "")                                          +
                middleName                                                 ,
                XmlParser.getString(c.getLastName())                      +
                (((firstName.length() > 0)  && (lastName.length() > 0))
                    ? " " : "")                                           +
                firstName                                                  ,
                lastName                                                  +
                (((firstName.length() > 0) && (lastName.length() > 0 ))
                    ? " " : "")                                           +
                firstName                                    ,
                firstName                                    ,
                lastName                                     ,
                middleName                                   ,
                p.getString(c.getTitle                  () ) ,
                p.getString(c.getOrganization           () ) ,
                p.getString(c.getHomeStreet             () ) ,
                p.getString(c.getHomeCity               () ) ,
                p.getString(c.getHomeState              () ) ,
                p.getString(c.getHomePostalCode         () ) ,
                p.getString(c.getHomeCountry            () ) ,
                p.getString(c.getOfficeStreet           () ) ,
                p.getString(c.getOfficeCity             () ) ,
                p.getString(c.getOfficeState            () ) ,
                p.getString(c.getOfficeCountry          () ) ,
                p.getString(c.getOfficePostalCode       () ) ,
                p.getString(c.getOtherStreet            () ) ,
                p.getString(c.getOtherCity              () ) ,
                p.getString(c.getOtherState             () ) ,
                p.getString(c.getOtherPostalCode        () ) ,
                p.getString(c.getOtherCountry           () ) ,
                p.getString(c.getOfficePhone            () ) ,
                p.getString(c.getHomePhone              () ) ,
                p.getString(c.getMobilePhone            () ) ,
                p.getString(c.getSecretaryPhone         () ) ,
                p.getString(c.getOfficeOtherPhone       () ) ,
                p.getString(c.getOfficeFax              () ) ,
                p.getString(c.getCallBackPhone          () ) ,
                p.getString(c.getMobileOtherPhone       () ) ,
                p.getString(c.getOrganizationPhone      () ) ,
                p.getString(c.getHomeOtherPhone         () ) ,
                p.getString(c.getHomeFax                () ) ,
                p.getString(c.getInternationalIsdnNumber() ) ,
                p.getString(c.getOtherPhone             () ) ,
                p.getString(c.getOtherFax               () ) ,
                p.getString(c.getPager                  () ) ,
                p.getString(c.getBusinessHomePage       () ) ,
                p.getString(c.getDepartment             () ) ,
                p.getString(c.getOffice                 () ) ,
                p.getString(c.getTitle                  () ) ,
                p.getString(c.getNickName               () ) ,
                p.getString(c.getSpouse                 () ) ,
                p.getString(c.getManager                () ) ,
                p.getString(c.getSecretary              () ) ,
                p.getString(c.getBusinessHomePage       () ) ,
                p.getString(c.getEmailAddress1          () ) ,
                p.getString(c.getEmailAddress2          () ) ,
                p.getString(c.getEmailAddress3          () ) ,
                p.getString(c.getFileas                 () )

              }
        );

        webDavHeaderMsg
            = MessageFormat.format(ItemDAO.WEBDAV_HEADER_PROPPATCH,
                new Object[] {
                    "/"                       +
                        server                +
                        "/"                   +
                        username              +
                        "/"                   +
                        resource              ,
                        c.getHref()
              }
        );


         try {
             response = this.webDavHttp.sendRequest(webDavHeaderMsg, credentials, webDavContactMsg, FILE_ENCODING);
         } catch (IOException e) {
             throw new DataAccessException("Error getting Exchange server response",
                                            e);
        }

        try {
            checkResponseStatus(response);
        } catch (Exception e) {

            throw new DataAccessException("USER "                       +
                                          username                      +
                                          " URI "                       +
                                          exchangeFolder                +
                                          " setting exchange contact"   ,
                                          e)                            ;

        }

        try {
            id = XmlParser.getRuidFromResponse(response);
        } catch (XmlParseException e) {
            throw new DataAccessException("Error getting resource id", e);
        }

        c.setId(id);

        return c;

    }

    /**
     * Delete a <i>Contact</i> from Exchange server.
     *
     * @param contact
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @throws sync4j.exchange.util.DataAccessException
     *
     */
    public void removeContact(Contact contact     ,
                              String  username    ,
                              String  credentials ,
                              String  exchangeFolder)
        throws DataAccessException {

        String webDavContactMsg        = null ;

        String webDavHeaderMsg         = null ;
        String response                = null ;

        String server                  = null ;
        String resource                = null ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        webDavContactMsg       = ""                      ;

        webDavHeaderMsg =
            MessageFormat.format(ItemDAO.WEBDAV_HEADER_REMOVE,
            new Object[] {
                        "/"                +
                        server             +
                        "/"                +
                        username           +
                        "/"                +
                        resource           ,
                        contact.getHref()
            }
        );


        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg, credentials, webDavContactMsg, FILE_ENCODING);
        } catch (IOException e) {
            throw new DataAccessException("Error getting Exchange server response",
                                          e);
        }

        try {
            checkResponseStatus(response);
        } catch (Exception e) {

            throw new DataAccessException("USER "                      +
                                          username                     +
                                          " URI "                      +
                                          exchangeFolder               +
                                          " removing exchange contact" ,
                                          e)                           ;

        }
    }

    /**
     * get contacts from Exchange Server
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     * @return array of find contacts
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact[] getContacts (String    username    ,
                                  String    credentials ,
                                  String[]  ids         ,
                                  String    exchangeFolder)
        throws DataAccessException {

        Contact[] exchangeContacts    = null ;

        String response               = null ;
        String webDavContactMsg       = null ;
        String webDavHeaderMsg        = null ;

        String server                 = null ;
        String resource               = null ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        String clause                 = null ;

        clause = XmlParser.getClause(ids);

        webDavContactMsg =
            MessageFormat.format(WEBDAV_MSG_SELECT_CONTACTS,
            new Object[] {server, username, resource, clause});

        webDavHeaderMsg =
            MessageFormat.format(ItemDAO.WEBDAV_HEADER_SELECT,

            new Object[] {
                        "/"       +
                        server    +
                        "/"       +
                        username  +
                        "/"       +
                        resource

            }
        );


         try {
             response = this.webDavHttp.sendRequest(webDavHeaderMsg, credentials, webDavContactMsg, FILE_ENCODING);
         } catch (IOException e) {
             throw new DataAccessException("Error getting Exchange server response",
                                           e);
         }

         try {
             checkResponseStatus(response);
         } catch (Exception e) {

            throw new DataAccessException("USER "                       +
                                          username                      +
                                          " URI "                       +
                                          exchangeFolder                +
                                          " getting Exchange contact"   ,
                                          e)                            ;

        }

        exchangeContacts = getContactsFromWebDavMsg(response);

        return exchangeContacts;

    }

     //------------------------------------------------------------- Private methods

    /**
     * Create a <i>Contact</i> array object from WebDav msg.
     *
     * @param id
     * @param webDavMsg
     *
     * @return a newly create <i>Calendar</i> object initialized with the
     *         fields in the result set
     *
     */
    private Contact[] getContactsFromWebDavMsg(String webDavMsg)
    throws DataAccessException {

        ArrayList  contacts   = null ;
        String []  resps      = null ;
        String []  msg        = null ;

        String     response   = null ;
        String     id         = null ;
        String     replUid    = null ;

        String     isFolder   = null ;

        int        count      = 0    ;

        msg = new String[] {webDavMsg};

        contacts = new ArrayList();

        try {

            resps = XmlParser.getXMLTag(msg, TAG_RESPONSE);

            for (int i = 0, l = resps.length; i < l; i++) {

                response    =  resps[i]                                        ;
                replUid     =  XmlParser.getXMLInitTagValue(response,
                                                        TAG_REPLUID)           ;

                id          =  XmlParser.getIdFromReplUid(replUid)             ;

                isFolder    =  XmlParser.getXMLInitTagValue(response,
                                                            TAG_IS_FOLDER)     ;

                if(PROP_NO_FOLDER.equals(isFolder)) {
                    contacts.add(getContactFromResponseTag(id, response));
                    count ++;
                }

            }

        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing contact", e);
        }

        return (Contact[])contacts.toArray(new Contact[count]);

    }

    /**
     * Create a <i>Contact</i> object from response tag.
     *
     * @param id
     * @param msgResponse
     *
     * @return a newly create <i>Contact</i> object initialized with the
     *         fields in the result set
     *
     */
    private Contact getContactFromResponseTag(String id, String msgResponse)
    throws DataAccessException {

        Contact c           = null ;
        String  lastUpdate  = null ;

        String  value       = null ;

        c = new Contact(id);

        try {

            c.setFirstName (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_FIRST_NAME
                )
            );
            c.setLastName (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_LAST_NAME
                )
            );
            c.setMiddleName (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_MIDDLE_NAME
                )
            );
            c.setTitle (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_TITLE
                )
            );
            c.setOrganization (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_ORGANIZATION
                )
            );

            c.setHomeStreet (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_HOME_STREET
                )
            );
            c.setHomeCity (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_HOME_CITY
                )
            );
            c.setHomeState (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_HOME_STATE
                )
            );
            c.setHomePostalCode (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_HOME_POSTAL_CODE
                )
            );
            c.setHomeCountry (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_HOME_COUNTRY
                )
            );

            c.setOfficeStreet (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_STREET
                )
            );
            c.setOfficeCity (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_CITY
                )
            );
            c.setOfficeState (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_STATE
                )
            );
            c.setOfficePostalCode (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_POSTALCODE
                )
            );
            c.setOfficeCountry (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_COUNTRY
                )
            );

            c.setOtherStreet (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_OTHER_STREET
                )
            );
            c.setOtherCity (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_OTHER_CITY
                )
            );
            c.setOtherState (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_OTHER_STATE
                )
            );
            c.setOtherPostalCode (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_OTHER_POSTALCODE
                )
            );
            c.setOtherCountry (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_OTHER_COUNTRY
                )
            );

            c.setOfficePhone (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_TELEPHONE_NUMBER
                )
            );
            c.setHomePhone (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_HOME_PHONE
                )
            );
            c.setMobilePhone (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_MOBILE
                )
            );
            c.setSecretaryPhone (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_SECRETARY_PHONE
                )
            );
            c.setOfficeOtherPhone (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_TELEPHONE_NUMBER2
                )
            );
            c.setOfficeFax (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_FAC_SIMILE_TELEPHONE_NUMBER
                )
            );
            c.setCallBackPhone (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_CALL_BACK_HOME
                )
            );
            c.setMobileOtherPhone (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_OTHER_MOBILE
                )
            );
            c.setOrganizationPhone (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_ORGANIZATION_MAIN_PHONE
                )
            );
            c.setHomeOtherPhone (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_HOME_PHONE2
                )
            );
            c.setHomeFax (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_HOME_FAX
                )
            );
            c.setInternationalIsdnNumber (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_INTERNATION_ISDN_NUMBER
                )
            );
            c.setOtherPhone (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_OTHER_TELEPHONE
                )
            );
            c.setOtherFax (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_OTHER_FAX
                )
            );
            c.setPager (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_PAGER
                )
            );

            c.setBusinessHomePage (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_BUSINESS_HOME_PAGE
                )
            );

            c.setDepartment (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_DEPARTMENT
                )
            );
            c.setOffice (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_ROOMNUMBER
                )
            );
            c.setProfession (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_PROFESSION
                )
            );
            c.setNickName (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_NICKNAME
                )
            );
            c.setSpouse (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_SPOUSECN
                )
            );
            c.setManager (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_MANAGER
                )
            );
            c.setSecretary (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_SECRETARY
                )
            );
            c.setBurl (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_BURL
                )
            );

            value =
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_EMAIL1
                );

            if (value != null && value.trim().length() > 0) {

                value = value.replaceAll("&lt;", "<");
                value = value.replaceAll("&gt;", ">");
                c.setEmailAddress1 (value.substring(value.indexOf("<") + 1,
                                                    value.indexOf(">")));
            } else {
                c.setEmailAddress1(value);
            }

            c.setEmailAddress2 (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_EMAIL2
                )
            );
            c.setEmailAddress3 (
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_EMAIL3
                )
            );
            c.setFileas(
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_FILEAS
                )
            );

            lastUpdate =
                XmlParser.getXMLInitTagValue(msgResponse,
                TAG_LAST_MODIFIED);

            c.setLastModified (
                XmlParser.webDavTagToDate (lastUpdate));

        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing contact item", e);
        }

        return c;

    }

}
